#include <GUIConstants.au3>
#include <GUIConstantsEx.au3>
#include <StaticConstants.au3>
#include <WindowsConstants.au3>
#include <WinAPI.au3>
#include "GUICreateTransparent.au3"
#include "GUICtrlPic.au3"

Global Const $SC_DRAGMOVE = 0xF012
_setTheme("whiteSimple")
_createGui($themeArr[0][1], $themeArr[0][2])

; test gui controls -------------------
$Input1 = GUICtrlCreateInput("Input1", 56, 112, 241, 21)
$Progress1 = GUICtrlCreateProgress(56, 152, 233, 17)
$Button1 = GUICtrlCreateButton("Button1", 56, 192, 105, 33)
$Button2 = GUICtrlCreateButton("Button2", 189, 192, 105, 33)
GUICtrlSetData($Progress1, 10)
; -------------------------------------

While 1
	$Msg = GUIGetMsg() ;
	Switch $Msg
		Case $GUI_EVENT_CLOSE
			Exit
		Case $gui_icon_close
			Exit
		Case $gui_icon_minimize
			GUISetState(@SW_MINIMIZE, $hGUI)
		Case $gui_icon_help
			MsgBox(-1, "Help", "you clicked the help button")
	EndSwitch
WEnd



; F U N C T I O N S ----------------------------------------------

; #FUNCTION# ====================================================================================================================
; Name...........: _setTheme
; Description ...: sets theme array from ini file in the folder $name to use the image files.
; Parameters ....: $name
; Return values .:
; ===============================================================================================================================
Func _setTheme($name)
	Global $themeArr[5][10]
	Local $themefolder = @ScriptDir & "\themes\" & $name & "\"
	Local $themeconf = $themefolder & "theme.ini"

	$themeArr[0][0] = IniRead($themeconf, "global", "name", $name)
	$themeArr[0][1] = IniRead($themeconf, "global", "width", "")
	$themeArr[0][2] = IniRead($themeconf, "global", "height", "")
	$themeArr[1][0] = $themefolder & IniRead($themeconf, "backgrounds", "640x480", "")
	$themeArr[2][0] = $themefolder & IniRead($themeconf, "icons", "close", "")
	$themeArr[2][1] = $themefolder & IniRead($themeconf, "icons", "maximize", "")
	$themeArr[2][2] = $themefolder & IniRead($themeconf, "icons", "minimize", "")
	$themeArr[2][3] = $themefolder & IniRead($themeconf, "icons", "help", "")
	$themeArr[3][0] = $themefolder & IniRead($themeconf, "iconshover", "close", "")
	$themeArr[3][1] = $themefolder & IniRead($themeconf, "iconshover", "maximize", "")
	$themeArr[3][2] = $themefolder & IniRead($themeconf, "iconshover", "minimize", "")
	$themeArr[3][3] = $themefolder & IniRead($themeconf, "iconshover", "help", "")
	$themeArr[4][0] = $themefolder & IniRead($themeconf, "iconsdisabled", "close", "")
	$themeArr[4][1] = $themefolder & IniRead($themeconf, "iconsdisabled", "maximize", "")
	$themeArr[4][2] = $themefolder & IniRead($themeconf, "iconsdisabled", "minimize", "")
	$themeArr[4][3] = $themefolder & IniRead($themeconf, "iconsdisabled", "help", "")
EndFunc   ;==>_setTheme

; #FUNCTION# ====================================================================================================================
; Name...........: _createGui
; Description ...: creates gui with main and child gui and theme pictures.
; Parameters ....: $width, $height
; Return values .:
; ===============================================================================================================================
Func _createGui($width, $height)
	; create transparent gui -----------------------------
	Global $hGUI = _GUICreate_Transparent($themeArr[1][0])

	; create child gui for controls ----------------------
	Global $hChild = GUICreate("MDIChild", $width, $height, -1, -1, $WS_POPUP, $WS_EX_MDICHILD + $WS_EX_LAYERED, $hGUI)
	GUISetBkColor(0xffffff)

	; set theme icons ------------------------------------
	Global $gui_icon_close = _GUICtrlPic_Create($themeArr[2][0], $width-40, 15, 30, 30, BitOR($SS_CENTERIMAGE, $SS_NOTIFY), Default)
	Global $gui_icon_maximize = _GUICtrlPic_Create($themeArr[4][1], $width-70, 15, 30, 30, BitOR($SS_CENTERIMAGE, $SS_NOTIFY), Default)
	Global $gui_icon_minimize = _GUICtrlPic_Create($themeArr[2][2], $width-100, 15, 30, 30, BitOR($SS_CENTERIMAGE, $SS_NOTIFY), Default)

	Global $gui_icon_help = _GUICtrlPic_Create($themeArr[2][3], $width-40, $height-40, 30, 30, BitOR($SS_CENTERIMAGE, $SS_NOTIFY), Default)

	; set background color of child gui transparent -----
	_WinAPI_SetLayeredWindowAttributes($hChild, 0xffffff, 0xff)
	GUISetState(@SW_SHOW, $hGUI)
	GUISetState(@SW_SHOW, $hChild)

	; set gui moveable ---------------------------------
	GUIRegisterMsg($WM_LBUTTONDOWN, "_WM_LBUTTONDOWN")
EndFunc   ;==>_createGui

; #FUNCTION# ====================================================================================================================
; Name...........: _WM_LBUTTONDOWN
; Description ...: need for drag window
; Parameters ....: $hWnd, $iMsg, $wParam, $lParam
; Return values .:
; ===============================================================================================================================
Func _WM_LBUTTONDOWN($hWnd, $iMsg, $wParam, $lParam)
	_SendMessage($hGUI, $WM_SYSCOMMAND, $SC_DRAGMOVE, 0)
EndFunc   ;==>_WM_LBUTTONDOWN